/***************************************************************************
                          PabiloPref.c  -  description
                             -------------------
    begin                : Mon Apr 15 2002
    copyright            : (C) 2002 by Michiel van Turnhout
    email                : info@qris.myweb.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "PabiloPref.h"
#include "Pabilo.h"
#include "Pabilo_res.h"

/***********************************************************************
 *
 * FUNCTION:    PabiloLoadPrefs
 *
 * DESCRIPTION: Load the preferences a
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
void PabiloLoadPrefs(PabiloPrefType *newprefs)
{
	PabiloPrefType  loadprefs;
	UInt16          prefsSize;
	Int16           prefsVersion;
	// Read the preferences / saved-state information.
	prefsSize = sizeof (PabiloPrefType);
	prefsVersion = PrefGetAppPreferences (sysFileCPabilo, pabiloPrefID, &loadprefs, &prefsSize, true);
	if (prefsVersion < pabiloPrefsVersionNum) {
    //Preferences not found set the defaults
		prefsVersion = noPreferenceFound;
		newprefs->whsz = 214;
		newprefs->maxspeed = 70;
		newprefs->updGraphSecs = 1;
		newprefs->updDataSecs = 3;
    newprefs->recordSecs = 30;
    newprefs->checkSpeedo = false;
	} else {
    //set the appropriate values
		newprefs->whsz = loadprefs.whsz;
		newprefs->maxspeed = loadprefs.maxspeed;
		newprefs->updGraphSecs = loadprefs.updGraphSecs;
		newprefs->updDataSecs = loadprefs.updDataSecs;
    newprefs->recordSecs = loadprefs.recordSecs;
    newprefs->checkSpeedo = loadprefs.checkSpeedo;
	}
  //Date format
  dateFormat = (DateFormatType) PrefGetPreference(prefDateFormat);
}

/***********************************************************************
 *
 * FUNCTION:    PabiloSavePrefs
 *
 * DESCRIPTION: Save the preferences
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 ***********************************************************************/
void PabiloSavePrefs(PabiloPrefType *newprefs) {
	PabiloPrefType  saveprefs;
	UInt32          uniqueID;

	saveprefs.whsz = newprefs->whsz;
  saveprefs.maxspeed = newprefs->maxspeed;
	saveprefs.updGraphSecs = newprefs->updGraphSecs;
	saveprefs.updDataSecs = newprefs->updDataSecs;
  saveprefs.recordSecs = newprefs->recordSecs;
  saveprefs.checkSpeedo = newprefs->checkSpeedo;

	PrefSetAppPreferences (sysFileCPabilo, pabiloPrefID, pabiloPrefsVersionNum, &saveprefs,sizeof(PabiloPrefType), true);
}

/***********************************************************************
 *
 * FUNCTION:	PreferencesHandleEvent
 *
 * DESCRIPTION:	Handles a Form event
 *
 * PARAMETERS:	event	pointer to an event structure
 *
 *
 * RETURNED:	returns handled/not handled
 *
 *
 * REVISION HISTORY:
 *			Name		Date					Description
 *			----		----					-----------

 *
 ***********************************************************************/

Boolean PreferencesHandleEvent(EventPtr event)
{
	FormPtr form;
	Boolean handled = false;
	ControlPtr		controlP;

	form = FrmGetActiveForm();

	switch (event->eType)
	{
		case frmOpenEvent:
			// Repaint form on open
			FrmDrawForm(form);
			WinDrawInt((Int32) prefs.whsz, 90,21);
			WinDrawInt((Int32) prefs.maxspeed, 90,33);
			WinDrawInt((Int32) prefs.updGraphSecs, 90,45);
			WinDrawInt((Int32) prefs.recordSecs, 90,57);
			WinDrawInt((Int32) prefs.updDataSecs, 90,69);
     	if (prefs.checkSpeedo) {
				controlP = FrmGetObjectPtr (form, FrmGetObjectIndex (form, spdConnID));
				CtlSetValue(controlP,1);
			}
			handled = true;
			break;
		case keyDownEvent:
			// commandKey was pressed (silk or hard)
			handled=false;
			break;

		case ctlSelectEvent:
			switch (event->data.ctlSelect.controlID)
			{
				case butOK:
					prefUpd = true;
					FrmReturnToForm(0);
					handled = true;
					// Free all data before we return to underlying form
					// Option might be to calculate the maxspeed with the wheelsize value
					break;
				case spdConnID:
					prefs.checkSpeedo = event->data.ctlSelect.on;
					handled = true;
					break;
			}
			break;
		case menuEvent:
			handled = true;
     break;

    case ctlRepeatEvent:
      switch (event->data.ctlRepeat.controlID)
      {
      case whszUP:
        prefs.whsz += 1;
        WinDrawInt((Int32) prefs.whsz, 90,21);
        handled = false;
        break;
      case whszDWN:
        if(prefs.whsz > 1)
          prefs.whsz -= 1;
        WinDrawInt((Int32) prefs.whsz, 90,21);
        handled = false;
        break;
      case maxspeedUP:
        if(prefs.maxspeed < 99) {
          prefs.maxspeed += 1;
          WinDrawInt((Int32) prefs.maxspeed, 90,33);
        }
        handled = false;
        break;
      case maxspeedDWN:
        if(prefs.maxspeed > 1)
          prefs.maxspeed -= 1;
          WinDrawInt((Int32) prefs.maxspeed, 90,33);
          handled = false;
        break;
      case scrupdrUP:
        prefs.updGraphSecs += 1;
        WinDrawInt((Int32) prefs.updGraphSecs, 90,45);
        handled = false;
        break;
      case scrupdrDWN:
        if(prefs.updGraphSecs > 1)
          prefs.updGraphSecs -= 1;
        WinDrawInt((Int32) prefs.updGraphSecs, 90,45);
        handled = false;
        break;
      case recordUP:
        prefs.recordSecs += 1;
        WinDrawInt((Int32) prefs.recordSecs, 90,57);
        handled = false;
        break;
      case recordDWN:
        if(prefs.recordSecs > 1)
          prefs.recordSecs -= 1;
        WinDrawInt((Int32) prefs.recordSecs, 90,57);
        handled = false;
        break;
      case dataUP:
        prefs.updDataSecs += 1;
        WinDrawInt((Int32) prefs.updDataSecs, 90,69);
        handled = false;
        break;
      case dataDWN:
        if(prefs.updDataSecs > 1)
          prefs.updDataSecs -= 1;
        WinDrawInt((Int32) prefs.updDataSecs, 90,69);
        handled = false;
        break;
      }
      break;

		default:
			break;
	}
	return handled;
}