/***************************************************************************
                          PabiloReplay.c  -  description
                             -------------------
    begin                : Tue Apr 23 2002
    copyright            : (C) 2002 by Michiel van Turnhout
    email                : info@qris.myweb.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*
 *OK I tried two approaches
 * 1 - draw the graph and redraw it at every table shift
 * 2 - at every table shift I shift the graph and draw the
 *     next line from the database records
 *     do this for every record even if we moved 20 records
 * option one looks blumpy and option two is to slow to be a serious Palm app worthy
 * Now I will think about another way of doing this and representing the graph
 * maybe scale down the graph and display it in one draw in some bitmap Handle and
 * then move a view over the bitmap.
 *
 * its all there in the PalmOS api I only need the memory in the device from the stack
 * The question is : is this stack sufficient for such opperations?
 * BmpCreate
 * WinCreateBitmapWindow
 * WinSetDrawWindow
 * WinClipRectangle
 * WinDrawBitmap
 * :-O Yes is works :-) create a bitmap and copy the part the scrollbar points to
 * It is real fast now scroll the table data and the Graph slide with it
 * move your stilus over the graph and it moves accordingly.
 * Tap on the graph and it returns to the start position
 *
 * Now solve the memory leak when we exit the app from the replay form
 * SOLVED memory leak: to make the event handler come to the frmCloseEvent have all events return false unless
 * they are realy handled in the event handler
 */
#include "PabiloReplay.h"
#include "Pabilo.h"
#include "Pabilo_res.h"
#include "PabiloDB.h"

UInt16 numRecords, recordIndex, TopRow ,OldTopRow = 0;
BitmapType *graphBitmap = NULL;
Int16 xpos = 0;

static void replayListDrawCell(MemPtr tableP, Int16 row, Int16 column, RectanglePtr bounds) {
	Int16			font;
	Char			DBName[32], *display_string, string[8];
	UInt16		TextLen;
	FontID		currFont;
	UInt32		type, creator, RowTop;
	UInt16		attributes,version,coltype,recordIndex;
	Boolean		b;
	UInt32      crDate,modDate,bckUpDate,modNum,numRecords,totalBytes,dataBytes;
	DateTimeType 	time;
  DateType  date;
  MemHandle       recordH;
  PabiloDBRecordPtr  recordP;
  RectangleType   tableBounds;
  
	RowTop = TblGetRowData(tableP, 1);
	recordIndex = RowTop + row;

  recordH = DmQueryRecord(PabiloDB,recordIndex);
  recordP = MemHandleLock(recordH);

	TimSecondsToDateTime (recordP->timeStamp,&time);
	// column 0 timestamp
	if( column ==0 ) {
		DateToAscii (time.month, time.day, time.year , dateFormat, string);
		display_string = string;
		TextLen = StrLen(display_string);
		font = 0;
	} else if( column ==1 ) {
//		TimeToAscii (time.hour, time.minute, dateFormat, string);
		TimSecondsToDateTime(recordP->timeStamp,&time);
		StrPrintF(string, "%2d:%2d:%2d", time.hour,time.minute,time.second);
		display_string = string;
		TextLen = StrLen(display_string);
		font = 0;
	} else if (column == 2 ) {
      DoubleToString(string ,recordP->speed,2);
			display_string = string;
			TextLen = StrLen(display_string);
  		font = 0;
  } else if (column == 3 ) {
      DoubleToString(string ,recordP->dist,2);
			display_string = string;
			TextLen = StrLen(display_string);
  		font = 0;
  }

  TblGetBounds(tableP,&tableBounds);
  if((row == 0) && (column > 0)) {
    WinSetForeColor(4);
		WinDrawLine(bounds->topLeft.x -1,bounds->topLeft.y,bounds->topLeft.x-1,tableBounds.extent.x);
		WinSetForeColor(15);
	}

	currFont = FntSetFont (font);
	if( (row % 2) != 0) {
		WinSetBackColor(2);
		WinFillRectangle(bounds,0);
	}
	WinPaintChars(display_string, TextLen, bounds->topLeft.x+2, bounds->topLeft.y);
	FntSetFont (currFont);			// Restore the font.
	WinSetBackColor(0);

  MemPtrUnlock(recordP);

	return;
}

static void drawReplayListForm(void) {
	FormPtr		    frm;
	TablePtr		  tableP;
	ScrollBarPtr	scrbP;
	UInt16			  dbIndex;
	Int16			    row, numRows;
	Int16			    currFont;
	Int8			    string[30],*n;

	frm = FrmGetActiveForm();
	tableP = FrmGetObjectPtr (frm, FrmGetObjectIndex (frm, replayTableID));
	TblHasScrollBar(tableP,true);
	numRows = TblGetNumberOfRows(tableP);
	scrbP = FrmGetObjectPtr (frm, FrmGetObjectIndex (frm, replayScrollID));
	if(numRecords < numRows) {
		SclSetScrollBar(scrbP,TopRow,0,0,numRows);
	} else {
		SclSetScrollBar(scrbP,TopRow,0,numRecords-numRows,numRows);
	}

	if (numRecords == 0) {
		FrmDrawForm (frm);
		return;
	}

	recordIndex = TopRow;
	if(TopRow > (numRecords-numRows))
		TopRow = numRecords - numRows;
	recordIndex = TopRow;
	TblSetRowData(tableP, 1, TopRow);
	for (row = 0; row < numRows; row++) {
		if ((recordIndex < numRecords) && (row < numRecords)) {
			TblSetItemStyle (tableP, row, 0, customTableItem);
			TblSetItemStyle (tableP, row, 1, customTableItem);
			TblSetItemStyle (tableP, row, 2, customTableItem);
			TblSetItemStyle (tableP, row, 3, customTableItem);
			TblSetRowUsable (tableP, row, true);
			TblMarkRowInvalid (tableP, row);
		}
		else  {
			TblSetRowUsable (tableP, row, false);
		}
		dbIndex++;
	}

	TblSetCustomDrawProcedure ((TablePtr) tableP, 0,(TableDrawItemFuncPtr) replayListDrawCell);
	TblSetCustomDrawProcedure ((TablePtr) tableP, 1,(TableDrawItemFuncPtr) replayListDrawCell);
	TblSetCustomDrawProcedure ((TablePtr) tableP, 2,(TableDrawItemFuncPtr) replayListDrawCell);
	TblSetCustomDrawProcedure ((TablePtr) tableP, 3,(TableDrawItemFuncPtr) replayListDrawCell);
	TblSetColumnUsable (tableP, 0, true);
	TblSetColumnUsable (tableP, 1, true);
	TblSetColumnUsable (tableP, 2, true);
	TblSetColumnUsable (tableP, 3, true);

	FrmDrawForm (frm);
	
}

//redraw the graph.
void drawReplayGraph() {
	RectangleType	     rec,savrec;
	UInt16             recindex;
	MemHandle          recordH;
	WinHandle          oldWinH,newWinH,savWinH;
	PabiloDBRecordPtr  recordP;
	double             newy,oldy,xpos;
	UInt16             error;
	UInt32             timeStamp;
	UInt32             startTime;
	double             avarage,  elapsedTime;
	double             dist = 0;
	char               *msg = "Please Wait...";
	
	rec.topLeft.x = 35; rec.topLeft.y = 70;
	rec.extent.x = 90; rec.extent.y = 20;

  WinSetDrawMode(winOverlay);
  WinGetFramesRectangle(boldRoundFrame, &rec,&savrec);
  savWinH = WinSaveBits(&savrec,&error);
	FntSetFont(2);  
  WinEraseRectangle(&rec,0);
  WinPaintRectangleFrame(boldRoundFrame, &rec);
  WinDrawChars(msg,StrLen(msg),rec.topLeft.x + 15, rec.topLeft.y +3);
	
	//The bitmap is sized by the number of records
	graphBitmap = BmpCreate(numRecords + 160,79,4,NULL,&error);
	newWinH = WinCreateBitmapWindow(graphBitmap,&error);
	oldWinH = WinSetDrawWindow(newWinH);
	recindex = 0;
	//walk through all records and draw the graph
	while(recindex < numRecords) {
		recordH = DmQueryRecord(PabiloDB,recindex);
		recordP = MemHandleLock(recordH);
		newy = 80 - recordP->speed;
		if((recindex == 0) || (recordP->dist < dist) ) {
			oldy=80;
			startTime = recordP->timeStamp;
			dist = recordP->dist;
		}
		if((recordP->timeStamp == startTime)) avarage = 0;
		else {
			elapsedTime = (double) (recordP->timeStamp - startTime) / 3600 ;
			avarage = (double) recordP->dist / elapsedTime;
		}
		WinSetForeColor(6);
		WinDrawLine(recindex,80,recindex+1,(80 - avarage));
		WinSetForeColor(15);
		newy = 80 - recordP->speed;
		WinDrawLine(recindex,oldy,recindex+1,newy);
		oldy = newy;

		MemPtrUnlock(recordP);
		recindex++;
	}
	WinSetDrawWindow(oldWinH);
	rec.topLeft.x = 0; rec.topLeft.y = 0;
	rec.extent.x = 160;rec.extent.y = 80;
	WinRestoreBits(savWinH,savrec.topLeft.x, savrec.topLeft.y);
	WinCopyRectangle(newWinH,oldWinH,&rec,0,81,winPaint);
	//delete the handles to free the memory associated with it.
	WinDeleteWindow(newWinH,true);
}

//We should scroll and add line to the left or to the right
//we should pass the number of records to scroll
void shiftReplayGraph(void) {
	WinHandle       destWinH,srcWinH;
	RectangleType	  rec;
	UInt16					error;
	UInt16          clipx;
  
	clipx = TopRow;
	if((clipx + 160) > numRecords) clipx = numRecords - 160;
	if(clipx < 0) clipx = 0;

	srcWinH = WinCreateBitmapWindow(graphBitmap,&error);
	destWinH = WinGetDrawWindow();
	rec.topLeft.x = clipx; rec.topLeft.y = 0;
	rec.extent.x = 160;rec.extent.y = 80;
	WinCopyRectangle(srcWinH,destWinH,&rec,0,81,winPaint);
	WinDeleteWindow(srcWinH,true);
}

static void HandlePopupSelect(EventPtr event)
{
	UInt8 *n;
	ListPtr 		listP = event->data.popSelect.listP;
	ControlPtr 	controlP = event->data.popSelect.controlP;

//	if (event->data.popSelect.listID == listID_selC1){
//		C1Type = LstGetSelection(listP);
//	} else if (event->data.popSelect.listID == listID_selC2) {
//		C2Type = LstGetSelection(listP);
//	}
//	n = listP->itemsText[listP->currentItem];
//	CtlSetLabel(controlP,n);

}

Boolean ReplayHandleEvent(EventPtr event)
{
	FormPtr         form;
	Boolean handled = false;
	MemHandle       recordH;
	PabiloDBRecordPtr  recordP;
	UInt16          recordIndex;
	DateTimeType    dateTime;
	Char	          timeString[timeStringLength];
	ListPtr         listP;
	UInt16          listIndex;
	RectangleType   r;


	form = FrmGetActiveForm();
	listIndex = FrmGetObjectIndex(form,replayActionList);
	
	switch (event->eType)
	{
		case frmOpenEvent:
			numRecords = DmNumRecords(PabiloDB);
			// Repaint form on open
			FrmDrawForm(form);
			drawReplayListForm();
			WinSetForeColor(8);
			WinDrawLine(0,80 ,160,80);
			WinSetForeColor(15);
			drawReplayGraph();
			shiftReplayGraph();
			handled = false;
			break;
		case frmCloseEvent:
			if(graphBitmap) {
				BmpDelete(graphBitmap);
				graphBitmap = NULL;
			}
			handled = false;
			break;
		case sclRepeatEvent:
			if( event->data.ctlSelect.controlID == replayScrollID) {
				TopRow = event->data.sclRepeat.newValue;
				shiftReplayGraph();
				drawReplayListForm();
			}
			handled = false;
			break;
		case keyDownEvent:
			// commandKey was pressed (silk or hard)
			handled=false;
			break;
		case ctlSelectEvent:
			switch (event->data.ctlSelect.controlID) {
			case butOK:
				//before we go back to the calling form clean up our mess
				if(graphBitmap) {
					BmpDelete(graphBitmap);
					graphBitmap = NULL;
				}
				FrmReturnToForm(0);
				handled = true;
				// Free all data before we return to underlying form
				// Option might be to calculate the maxspeed with the wheelsize value
				break;
			}
			break;
		case tblSelectEvent:
			TblUnhighlightSelection(event->data.tblSelect.pTable);
			TblGetItemBounds(event->data.tblSelect.pTable,event->data.tblSelect.row,event->data.tblSelect.column, &r);

			FrmHideObject(form,listIndex);
			FrmDrawForm(form);

			listP = (ListPtr) FrmGetObjectPtr(form,listIndex );
			LstSetPosition(listP,r.topLeft.x,r.topLeft.y);
			FrmShowObject(form,listIndex);
//			handled = true;
			break;
		case lstSelectEvent:
			FrmHideObject(form,listIndex);
			HandlePopupSelect(event);
//			handled = true;
			break;
		case penUpEvent:
			if( generalFlag ) {
				Int16 move = 0;
				Int16 ntr = 0;
				if(xpos > event->screenX) {
					move = xpos - event->screenX;
					ntr = (Int16)TopRow + move;
				}
				else if(event->screenX > xpos) {
					move = event->screenX - xpos;
					ntr = (Int16)TopRow - move;
				}
				// if we tap the graph the following is true and the graph returns to the start
				// position
				// if(xpos == event->screenX) { move = 0;  ntr = 0; }
				if(ntr < 0) TopRow = numRecords - 160;
				else if(ntr > numRecords) TopRow = 0;
				else TopRow = (UInt16) ntr;
				shiftReplayGraph();
				drawReplayListForm();
				generalFlag = false;;
			}
			break;
		case penDownEvent:
			if(event->screenY > 80) { 
				xpos = event->screenX;
				generalFlag = true;
			} else generalFlag = false;
			break;
		case menuEvent:
			handled = true;
			break;

		default:
			break;
	}
	return handled;
}