/***************************************************************************
                          PabiloTrip.c  -  description
                             -------------------
    begin                : Fri Jan 17 2003
    copyright            : (C) 2003 by Michiel van Turnhout
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************
 *
 *   As you have guesed I want a new database for every trip.
 *   Thats why we need trip management here you select a database to work with
 *   because it is the database that represents the trip.
 *   So her we select or define a new database name. then when open is pressed
 *   we check if a database was selected and cal PabiloGetDatabase to initialize
 *   the PabiloDB pointer.
 *
 ***************************************************************************/
#include <PalmOS.h>

#include "Pabilo.h"
#include "PabiloTrip.h"
#include "PabiloDB.h"
#include "Pabilo_res.h"


UInt16 topRow = 0;
UInt16 numDB ;

Char *   dbNames[MAXTRIPS];


static void tripListDrawCell(MemPtr tableP, Int16 row, Int16 column, RectanglePtr bounds) {
	Int16			font;
	Char			DBName[32], *display_string = NULL, *none = " ";
	UInt16		TextLen;
	FontID		currFont;
	UInt32		type, creator, RowTop;
	UInt16		attributes,version,coltype,recordIndex;
	Boolean		b;
	UInt32      crDate,modDate,bckUpDate,modNum,numRecords,totalBytes,dataBytes;
	DateTimeType 	time;
  DateType  date;
  RectangleType   tableBounds;

	RowTop = TblGetRowData(tableP, 1);
	// column 0 dbName
	if( column ==0 ) {
		
		display_string = dbNames[row + topRow];
		TextLen = StrLen(display_string);
		font = 0;
	} 
	if( column ==1 ) {

		display_string = none;
		TextLen = StrLen(display_string);
		font = 0;
	}
	if( column ==2 ) {

		display_string = none;
		TextLen = StrLen(display_string);
		font = 0;
	}
	if( column ==3 ) {

		display_string = none;
		TextLen = StrLen(display_string);
		font = 0;
	}

  TblGetBounds(tableP,&tableBounds);
  if((row == 0) && (column > 0)) {
    WinSetForeColor(4);
		WinDrawLine(bounds->topLeft.x -1,bounds->topLeft.y,bounds->topLeft.x-1,tableBounds.extent.x);
		WinSetForeColor(15);
	}

	currFont = FntSetFont (font);
	if( (row % 2) != 0) {
		WinSetBackColor(2);
		WinFillRectangle(bounds,0);
	}
	WinPaintChars(display_string, TextLen, bounds->topLeft.x+2, bounds->topLeft.y);
	FntSetFont (currFont);			// Restore the font.
	WinSetBackColor(0);

	return;
}


void InitTripForm(void) {
	LocalID           dbID = NULL;
	UInt16            cardNo = 0;
	Err               nextDBErr = errNone;
	Err               error;
	Int16             numRows,row;
	DmSearchStateType stateInfo;
	Boolean           restartSearch = true;
	Char *            dbName;
	
	numDB = 0;topRow = 0 ;
	while(nextDBErr == errNone) {
		nextDBErr = DmGetNextDatabaseByTypeCreator(restartSearch,&stateInfo,pabiloDBType,sysFileCPabilo,false,&cardNo,&dbID);
		if(dbID) {
			dbName = (Char *) MemPtrNew(TRIPNAMELENGTH + 1);
			error = DmDatabaseInfo(cardNo,dbID,dbName,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
			dbNames[numDB] = dbName;
			numDB++;
		}
		restartSearch = false;
	}
}

void ResetTripForm(void) {
	UInt16	count;
	//Free the memory for the trip names
	for(count = 0; count < numDB; count++) {
		MemPtrFree(dbNames[count]);
		dbNames[count] = NULL;
	}
	numDB = 0;
}

void DrawDBList(void) {
	FormPtr           form;
	TablePtr          tableP;
	ScrollBarPtr      scrbP;
	Err               error;
	Int16             numRows,row;
	DmSearchStateType stateInfo;
	Boolean           restartSearch = true;
	
	form = FrmGetActiveForm();
	tableP = FrmGetObjectPtr (form, FrmGetObjectIndex (form, tripTableID));
	TblHasScrollBar(tableP,true);
	numRows = TblGetNumberOfRows(tableP);
	//Scrollbar init
	scrbP = FrmGetObjectPtr (form, FrmGetObjectIndex (form, tripScrollID));
	if(numDB < numRows) {
		SclSetScrollBar(scrbP,topRow,0,0,numRows);
	} else {
		SclSetScrollBar(scrbP,topRow,0,numDB-numRows,numRows);
	}
	for(row = 0; row < numRows; row++) {
		TblSetItemStyle (tableP, row, 0, customTableItem);
		TblSetItemStyle (tableP, row, 1, customTableItem);
		TblSetItemStyle (tableP, row, 2, customTableItem);
		TblSetItemStyle (tableP, row, 3, customTableItem);
		if(row < numDB) {
			TblSetItemPtr(tableP,row,0,(Char *)dbNames[row + topRow]);
			TblMarkRowInvalid (tableP, row);
			TblSetRowUsable(tableP,row,true);
		} else {
			TblSetRowUsable(tableP,row,false);
		}
	}
	TblSetCustomDrawProcedure ((TablePtr) tableP, 0,(TableDrawItemFuncPtr) tripListDrawCell);
	TblSetCustomDrawProcedure ((TablePtr) tableP, 1,(TableDrawItemFuncPtr) tripListDrawCell);
	TblSetCustomDrawProcedure ((TablePtr) tableP, 2,(TableDrawItemFuncPtr) tripListDrawCell);
	TblSetCustomDrawProcedure ((TablePtr) tableP, 3,(TableDrawItemFuncPtr) tripListDrawCell);
	TblSetColumnUsable (tableP, 0, true);
	TblSetColumnUsable (tableP, 1, true);
	TblSetColumnUsable (tableP, 2, true);
	TblSetColumnUsable (tableP, 3, true);
}

void RedrawTripForm(FormPtr form) {
	ResetTripForm();
	InitTripForm();
	DrawDBList();
	FrmDrawForm(form);
	generalFlag = false;
}

/***********************************************************************
 *
 * FUNCTION:	TripHandleEvent
 *
 * DESCRIPTION:	Handles a Form event
 *
 * PARAMETERS:	event	pointer to an event structure
 *
 *
 * RETURNED:	returns handled/not handled
 *
 *
 * REVISION HISTORY:
 *			Name		Date					Description
 *			----		----					-----------

 *
 ***********************************************************************/

Boolean TripHandleEvent(EventPtr event)
{
	FormPtr form;
	Boolean handled = false;
	ControlPtr		controlP;
	UInt16 count;

	form = FrmGetActiveForm();

	if(generalFlag) RedrawTripForm(form);

	switch (event->eType)
	{
		case frmOpenEvent:
			// Repaint form on open
			RedrawTripForm(form);
			if(numDB > 40) FrmAlert(AlertMaxTripsID);
			handled = true;
			break;
		case frmCloseEvent:
			ResetTripForm();
			handled = false;
			break;
		case tblSelectEvent:
			if(pabiloDBName) { MemPtrFree(pabiloDBName);
				pabiloDBName = NULL;
			}
			pabiloDBName = (Char *)MemPtrNew(StrLen(dbNames[topRow + event->data.tblSelect.row]) +1);
			StrCopy( pabiloDBName,dbNames[topRow + event->data.tblSelect.row]);
			handled = true;
			break;
		case keyDownEvent:
			// commandKey was pressed (silk or hard)
			handled=false;
			break;
		case sclRepeatEvent:
			if( event->data.ctlSelect.controlID == tripScrollID) {
				topRow = event->data.sclRepeat.newValue;
				DrawDBList();
				FrmDrawForm(form);
			}
			handled = false;
			break;
		case ctlSelectEvent:
			switch (event->data.ctlSelect.controlID) {
			case butOpen:
				//This for testing the open database functions.
				//pabiloDBName = "PabiloDB";
				if(pabiloDBName) {
					ResetTripForm();
					FrmGotoForm(frmMain);
				} else {
					FrmAlert(AlertTripNameID);
				}
				handled = true;
				break;
			case butNew:
				//Delete the old memory allocation
				if(pabiloDBName) {
					MemPtrFree(pabiloDBName);
					pabiloDBName = NULL;
				}
				FrmPopupForm(frmNewTrip);
				generalFlag = true;
				handled = true;
				break;
			case butDelete:
				if(pabiloDBName) {
					PabiloDeleteDB(pabiloDBName);
					if(pabiloDBName) {
						MemPtrFree(pabiloDBName);
						pabiloDBName = NULL;
					}
					//indicate a redraw.
					generalFlag = true;
				} else {
					FrmAlert(AlertTripNameID);
				}
				handled = true;
				break;
			}
			handled = true;
			break;
		case menuEvent:
			handled = true;
			break;

    case ctlRepeatEvent:
			switch (event->data.ctlRepeat.controlID) {
			}
			break;
		default:
			break;
	}
	return handled;
}

/***********************************************************************
 *
 * FUNCTION:	NewTripHandleEvent
 *
 * DESCRIPTION:	Handles a Form event
 *
 * PARAMETERS:	event	pointer to an event structure
 *
 *
 * RETURNED:	returns handled/not handled
 *
 *
 * REVISION HISTORY:
 *			Name		Date					Description
 *			----		----					-----------

 *
 ***********************************************************************/

Boolean NewTripHandleEvent(EventPtr event)
{
	FormPtr       form;
	Boolean       handled = false;
	ControlPtr    controlP;
	FieldPtr      nameField;
	MemHandle     h;
	Char *        s;
	Err           error;
	UInt16        mode;

	form = FrmGetActiveForm();

	switch (event->eType)
	{
		case frmOpenEvent:
			// Repaint form on open
			FrmDrawForm(form);
			handled = true;
			break;
		case ctlSelectEvent:
			switch (event->data.ctlSelect.controlID) {
			case butOK:
				//if a database exists do not create accept the db name
				//First get a pointer to the field resource
				nameField = (FieldPtr) FrmGetObjectPtr(form,FrmGetObjectIndex(form,tripnameField));
				if( FldGetTextLength(nameField)) {
					//get a handle to the structure
					h = FldGetTextHandle(nameField);
					//Disconnect the handle from the field
					FldSetTextHandle(nameField,NULL);
					//Lock the handle and make a Char pointer of it
					s = (Char *) MemHandleLock(h);
					if(! DBExists(s)) {
						mode = dmModeReadWrite;
						error = PabiloGetDatabase(&PabiloDB, s ,mode);
						DmCloseDatabase(PabiloDB);
						PabiloDB=NULL;
					} 
					//unlock the handle and free the memory associated with it
					MemHandleUnlock(h);
					MemHandleFree(h);
				}
				FrmReturnToForm(0);
				handled = true;
				break;
			case butCancel:
				FrmReturnToForm(0);
				handled = true;
				break;
			}
			handled = true;
			break;
		default:
			break;
	}
	return handled;
}
 