/***************************************************************************
                          frmMain.c
                             -------------------
    begin                : Fri Jul 20 2001
    copyright            : (C) 2001 by Michiel van Turnhout
    email                : info@qris.myweb.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "Pabilo.h"
#include "PabiloDB.h"
#include "PabiloPref.h"
#include "battery.h"

/***********************************************************************
 * Global variables for this module
 **********************************************************************/
UInt32    startTime;
double    kmph = 0.0, dist = 0.0, average = 0.0;
UInt8     oldY=160;           //needed to draw the linebetween newY and oldY
TimeFormatType timeFormat;
UInt8     oldpercent=0; //needed to determine if battery update is needed.
static    UInt8  xgrid = 1;   //needed to determine if the grid should be drawn
Boolean   prefUpd = true;

//PabiloII
PabiloControl ctrl;
PabiloTiming  tmr;

//For debugging set to false
//Boolean connSpeedo = false;

//prototypes
void drawMainForm(FormUpdateReason reason);
void updateGraph(void);

//PabiloII get data function
// an explenation of the calculation
//    val1 = ((double) whsz / 100000);
//    val2 = ((double) timerValue * 0.0000925943);
//    val3 = ((double) val2 / 3600);
//    kmph = ((double) val1 / val3);
Boolean getSpeedoData(void) {
  Err       error;
  Boolean   retval=true;
  UInt16    portID;
  UInt16    spdsp = 1;
  UInt16    timerValue = 1;
  UInt32    wheelRounds = 0;
  // use the buff values to debug with the emulator
  Char      buff1[]  = "00000000";
  Char      buff2[]  = "ffff";

  SrmOpen(U328,BAUDRATE,&portID);
  SrmSend(portID,"g",1,&error);
  SrmSendWait(portID);
  //keep the timeout as short as possible to avoid device freeze!
  //first speed
  SrmReceive(portID,buff2,4,SERIALTIMEOUT,&error);
  //then wheelrounds
  SrmReceive(portID,buff1,8,SERIALTIMEOUT,&error);
  SrmReceiveFlush(portID,SERIALTIMEOUT);
  SrmSendFlush(portID);
  SrmClose(portID);
  if((prefs.checkSpeedo) && (error != 0))
    retval=false;
  timerValue = (UInt16) StrHToI(buff2);
  wheelRounds = StrHToI(buff1);
// FOR DEBUG : see what came in after converion
//  FntSetFont(0);
//  StrIToH(buff1,timerValue);
//  WinDrawChars(buff1,StrLen(buff1),110,25);

// Calculate the new values with the new data
  if(timerValue == MAXTIMERVAL)
    kmph = 0;
  else {
    kmph = ((double) prefs.whsz / 100000) /  ((double) ((double) timerValue * 0.0000925943) / 3600);
  }
  dist = ((double)prefs.whsz / 100000) * wheelRounds;
  return retval;
}

//PabiloII get data function
int resetSpeedo(void) {
  Err       error;
  UInt16    portID;

  SrmOpen(U328,BAUDRATE,&portID);
  SrmSend(portID,"r",1,&error);
  SrmSendWait(portID);
  //the folowing gives the palm the time to transmit before I close the port.
  error = SrmReceiveWait(portID,1,SERIALTIMEOUT);
  SrmSendFlush(portID);
  SrmClose(portID);
  startTime = TimGetSeconds();
//  tmr.startTics = TimGetTicks();
}

/***********************************************************************
 *
 * FUNCTION:	checkForUpdate
 *
 * DESCRIPTION:	This routine checks several things and decides wether to
 *              update the display
 *
 * PARAMETERS:	none
 *					
 * RETURNED:	  none
 *
 * REVISION HISTORY:
 *			Name		Date					Description
 *			----		----					-----------
 *      MVT      14-04-2002   added visible menus check
 *      MVT      24-05-2002   Merged the checkForRecordUpdate function
 ***********************************************************************/

void checkForUpdate() {
  UInt32          tm;
  PabiloItemType  newRecord;
  Boolean         menuVisible = false;
  MenuBarType     *menuBarTypeP = MenuGetActiveMenu();

  if (menuBarTypeP) menuVisible = menuBarTypeP->attr.visible;
  if(!menuVisible) {  // no visible menus, so it s OK to update the display
                      // this check should be done for iconbar as well
    tm = TimGetSeconds();
  	if ( prefs.updGraphSecs <= ( tm - ctrl.doGraph) ) {
      ctrl.doGraph = tm;
  		updateGraph();
  	}
  	if ( prefs.updDataSecs <= ( tm - ctrl.doData) ) {
  		drawMainForm(updateValues);
  		ctrl.doData = tm;
  	}
    if(prefs.recordSecs <= (tm - ctrl.doRecord) ) {
      // set doRecord for next value
      ctrl.doRecord = tm;
      // Create new Record with the current values
      newRecord.speed=kmph;
      newRecord.timeStamp = tm;
      newRecord.dist=dist;
      PabiloNewRecord(PabiloDB,&newRecord);
    }
    if(prefUpd == true) {
      DrawGridLabels();
      prefUpd = false;
    }
  }
}

/***********************************************************************
 *
 * FUNCTION:	DrawGridLabels
 *
 * DESCRIPTION:	This routine draws the grid labels with use of the
 *              preference values
 *
 * PARAMETERS:	uses global preference structure
 *					
 * RETURNED:	  nothing
 *
 * REVISION HISTORY:
 *			Name		Date					Description
 *			----		----					-----------
 *      MVT      20-07-2001   creation
 ***********************************************************************/
void DrawGridLabels(void) {
  Char  string[3];
  FntSetFont(0);
  StrIToA(string,prefs.maxspeed);
  WinDrawChars(string,StrLen(string),0,MAXY - 5 );
  StrIToA(string,prefs.maxspeed/2);
  WinDrawChars(string,StrLen(string),0,((MINY - MAXY)/2) + MAXY - 5);
}

/***********************************************************************
 *
 * FUNCTION:	drawMainForm
 *
 * DESCRIPTION:	This function fills the variable fields in the mainform
 *              and it draws some graphic for batery level indication
 *
 * PARAMETERS:	uses Global vars  : kmph dist batVoltage
 *					
 * RETURNED:	  none
 *
 * REVISION HISTORY:
 *			Name		Date					Description
 *			----		----					-----------
 *      MVT      20-07-2001   creation
 ***********************************************************************/

void drawMainForm(FormUpdateReason reason) {
	UInt32          tm;
	DateTimeType    dateTime;
	RectangleType	  rec;
 	Char	          string[20];
  Char            badkmph[]="..........", maxmesg[]="MAX speed";
  Boolean         pluggedin;
 	UInt8				    percent;
	RectangleType		r1, r2, save;
	Coord				    full, level, x, y;
	MemHandle			  h;
	FontID		      currFont;
		
  FormPtr form = FrmGetActiveForm();

  // Draw lines and Pabilo text
  if(reason == initForm) {
		//To redraw the battery just set oldpercent to 0
		oldpercent = 0;
    FntSetFont(0);
    WinDrawChars(pabiloDBName, StrLen(pabiloDBName), 105, 24);
    WinSetForeColor(8);
  	currFont = FntSetFont (1);
    FntSetFont(1);
    WinSetTextColor(8);
    WinDrawChars("Pabilo", 6, MINX+10, 100);
		WinDrawChars("2002", 4, MINX+13, 110);
		WinDrawChars("QRIS", 4, MINX+13, 120);
    WinSetTextColor(15);
  	FntSetFont (currFont);			// Restore the font.
		WinDrawLine(0,MAXY - 12,160,MAXY - 12);
    WinSetForeColor(15);
  }

  if(reason == initValues) {
    //set time in doRecord to check the past seconds
    tmr.startTics = tmr.oldTics = TimGetTicks();
    ctrl.doData = ctrl.doRecord = ctrl.doGraph = startTime = TimGetSeconds();
		timeFormat = tfColon24h;
  }

	tm = TimGetSeconds();
	TimSecondsToDateTime(tm,&dateTime);

// Speed
	rec.topLeft.x = 0	; rec.topLeft.y = 0;
	rec.extent.x = 80; rec.extent.y = 35;
	WinEraseRectangle(&rec,0);

	FntSetFont(129);
  if(kmph > prefs.maxspeed) {
	  WinDrawChars(badkmph, StrLen(badkmph), 0, 1);
	  FntSetFont(2);
    WinDrawChars(maxmesg, StrLen(maxmesg), 0, 13);
  }
  DoubleToString(string, kmph, 1);
	WinDrawChars(string, StrLen(string), 0, 1);

// Time
	FntSetFont(2);
	TimeToAscii(dateTime.hour,dateTime.minute,timeFormat,string);
	WinDrawChars(string, StrLen(string), 25, 38);
//distance
  WinDrawDouble(dist,2,118,1);
//average
	WinDrawDouble(average,1,118,12);

//  percent =25; // DEBUG
	if (FrmVisible(form) ) {
		WinDrawBattery();
	}
}

// calculate the Y coordinate for the graph
double calcCoord(double val) {
  double retval;
  retval = 160 - ((val/prefs.maxspeed) * (160 - MAXY));
  if(retval < MAXY)
    retval = MAXY;
  return retval;
}

/***********************************************************************
 *
 * FUNCTION:	updateGraph
 *
 * DESCRIPTION:	This function updates the graphic by first
 *              scrolling the graph to the left and then connecting
 *              the new line.
 *
 * PARAMETERS:	uses Global vars      : oldY kmph
 *					
 * RETURNED:	  modifies global vars  : oldY
 *
 * REVISION HISTORY:
 *			Name		Date					Description
 *			----		----					-----------
 *      MVT      20-07-2001   creation
 *
 * TODO:
 *      Add a grid and labels for every 30 pixels
 ***********************************************************************/

void updateGraph() {
	RectangleType	rec;
	WinDirectionType direction;
	UInt32          tm,elaps,newTics;
	DateTimeType    dateTime;
 	Char	          timeString[timeStringLength];
  double          newY,elapsedTime;

  //Here the connection is checked according to prefs values
  if(!getSpeedoData()) {
    UInt16 resp = FrmAlert(AlertSpeedoID);
    if(resp == 1) {
			//Disable check in prefs.
			prefs.checkSpeedo = false;
		}
  }
  
//  calcVars();
	rec.topLeft.x = MINX	; rec.topLeft.y = 60;
	rec.extent.x = MAXX   ; rec.extent.y = 100;
	direction = winRight;
	WinScrollRectangle ( &rec, direction, 1, &rec);

  //every graph update recalculate the average speed
  newTics = TimGetTicks();
  newTics -= tmr.startTics;
  elapsedTime = (double) (newTics / tps) / 3600; // in seconds
  average = (double) dist / elapsedTime;
  WinSetForeColor(6);
	WinDrawLine(MINX+1,MINY,MINX+1,calcCoord(average));

  //Draw the vertical grid line right now
  if(xgrid-- == 0) {
    WinSetForeColor(4);
    WinDrawLine(MINX+1,MINY,MINX+1,MAXY);	
    tm = TimGetSeconds();
	  TimSecondsToDateTime(tm,&dateTime);
	  TimeToAscii(dateTime.hour,dateTime.minute,timeFormat,timeString);
  	FntSetFont(0);
  	WinDrawChars(timeString, StrLen(timeString), MINX+1, MAXY - 11);
    xgrid = UPDGRID;
    WinSetForeColor(15);
  }
	//horizontal grid  should get drwan her to avoid that the average graph overwrites it
	WinSetForeColor(4);
	WinDrawLine(MINX,MAXY,MINX+1,MAXY);
	WinDrawLine(MINX,((MINY - MAXY)/2) + MAXY,MINX+1,((MINY - MAXY)/2) + MAXY);
  // do something smart to draw between 160 and 70
  // kmph/maxspeed * 70
  WinSetForeColor(15);
  newY = calcCoord(kmph);
	WinDrawLine(MINX+2,oldY,MINX+1,newY);
	oldY = newY;
}

/***********************************************************************
 *
 * FUNCTION:	MainFormHandleEvent
 *
 * DESCRIPTION:	Handles a Form event
 *
 * PARAMETERS:	event	pointer to an event structure
 *              global vars
 *					
 * RETURNED:	returns handled/not handled
 *            updates global vars :
 *
 * REVISION HISTORY:
 *			Name		Date					Description
 *			----		----					-----------

 * 
 ***********************************************************************/

Boolean MainFormHandleEvent(EventPtr event)
{
	UInt16  mode;
	FormPtr form;
	Boolean handled = false;
  Err error;

	switch (event->eType) {
		case frmOpenEvent:
			// Find the application's data file.  If it doesn't exist, create it.
			mode = dmModeReadWrite;
			error = PabiloGetDatabase(&PabiloDB, pabiloDBName ,mode);
			if (error)
				return false;
			// Repaint form on open
      // change to 4bpp grayscale
			form = FrmGetActiveForm();
      //FrmDrawForm(generates an event that is handled afterwards.
			FrmDrawForm(form);
      drawMainForm(initValues);
      drawMainForm(initForm);
			drawMainForm(redrawForm);
      //setting the prefUpd forces Grid label redraw
      prefUpd = true;
			handled = true;
			break;
		case frmCloseEvent:
			// Close the application's data file.
			if(PabiloDB) {
				DmCloseDatabase(PabiloDB);
				PabiloDB = NULL;
			}
		break;
    case frmUpdateEvent:
      DrawGridLabels();
      handled = true;
      break;
	  case ctlSelectEvent:
			switch (event->data.ctlSelect.controlID) {
				case replayBut:
					FrmPopupForm (frmReplay);
					handled = true;
					break;
				case butOK:
					FrmGotoForm(frmTrip);
					handled = true;
					break;
			}
		  break;
    case menuEvent:
      switch (event->data.menu.itemID) {
        case  menuitemID_Prefs:
          FrmPopupForm (frmPrefs);
          handled = true;
          break;
        case menuitemID_Manage:
          FrmGotoForm (frmTrip);
          handled = true;
          break;
        case menuitemID_Open:
          FrmPopupForm (frmReplay);
          handled = true;
          break;
        case menuitemID_Reset:
          resetSpeedo();
          handled = true;
          break;
        case  menuitemID_About:
          FrmPopupForm (frmAbout);
          handled = true;
          break;
        default:
          break;
      }
      break;
    case menuCmdBarOpenEvent:
      //this is to suppress the menucommandbar
      handled=true;
      break;
    case nilEvent:
//    the updater
      checkForUpdate();
      break;
  }
	return handled;
}

/***********************************************************************
 *
 * FUNCTION:	AboutHandleEvent
 *
 * DESCRIPTION:	Handles a Form event
 *
 * PARAMETERS:	event	pointer to an event structure
 *
 *					
 * RETURNED:	returns handled/not handled
 *
 *
 * REVISION HISTORY:
 *			Name		Date					Description
 *			----		----					-----------

 *
 ***********************************************************************/

Boolean AboutHandleEvent(EventPtr event)
{
	FormPtr form;
	Boolean handled = false;

	switch (event->eType)	{
		case frmOpenEvent:
			// Repaint form on open
			form = FrmGetActiveForm();
			FrmDrawForm(form);
			handled = true;
			break;
  	case ctlSelectEvent:
  		switch (event->data.ctlSelect.controlID) {
			case butOK:
				FrmReturnToForm(0);
				handled = true;
				break;
  		}
  		break;
  }
	return handled;
}