/***************************************************************************
                          main.c
                             -------------------
    begin                : Fri Jul 20 2001
    copyright            : (C) 2001 by Michiel van Turnhout
    email                : info@qris.myweb.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#include <PalmOS.h>

#include "Pabilo.h"
#include "Pabilo_res.h"
#include "PabiloTrip.h"
#include "PabiloPref.h"
#include "PabiloReplay.h"
#include "PabiloDB.h"

UInt16    autoOffTime;
UInt16    tps;

MemHandle fontH;

//Application prefs in a global structure
PabiloPrefType prefs;

//Pabilo database reference
Char * pabiloDBName = NULL;
DmOpenRef			PabiloDB = NULL;

//Date format
DateFormatType dateFormat;

Boolean generalFlag = false;


/***********************************************************************
 *
 * FUNCTION:	StartApplication
 *
 * DESCRIPTION:	This routine will launch the application's main form
 *
 * PARAMETERS:	 none
 *					
 * RETURNED:	 returns nothing
 *
 * REVISION HISTORY:
 *			Name		Date					Description
 *			----		----					-----------
 *
 ***********************************************************************/

static int StartApplication(void)
{
	Err     err = 0;
	UInt16  attr;
	UInt32  uniqueID;
	UInt32  depth;
	FontPtr font129;
	Boolean colorMode;

	//The palm should not auto shutdown
	autoOffTime = SysSetAutoOffTime(0);
	// Get a handle to the custom font resource.
	// Note that 'CustomFont' is the custom font's ID.
	fontH = DmGetResource('NFNT', CustomFont);
	// Lock the handle
	font129 = MemHandleLock(fontH);
	// Define the font.
	// Note that this function only works with Palm OS 3.0 or higher, and 
	// that '129' is the custom font's 'FontID'.
	FntDefineFont(129, font129);

	// device specific clock speed
	tps = SysTicksPerSecond();  // 100 on palm vx

	PabiloLoadPrefs(&prefs);

	WinScreenMode(winScreenModeGetDefaults,NULL,NULL,&depth,&colorMode);
//	WinScreenMode(winScreenModeGetSupportedDepths,NULL,NULL,&depth,NULL);
//	WinScreenMode(winScreenModeGetSupportsColor,NULL,NULL,NULL,&colorMode);
	depth=4;
	WinScreenMode(winScreenModeSet,NULL,NULL,&depth,&colorMode);

	FrmGotoForm(frmTrip);
	return err;
}


/***********************************************************************
 *
 * FUNCTION:	ApplicationHandleEvent
 *
 * DESCRIPTION:	This routine is called from the event loop, and its
 *				main responsibility is to load forms and set form
 *				eventhandlers.
 *
 * PARAMETERS:	 EventPtr	event	Pointer to the event
 *					
 * RETURNED:	 handled/not handled
 *
 * REVISION HISTORY:
 *			Name		Date					Description
 *			----		----					-----------
 *
 ***********************************************************************/

static Boolean ApplicationHandleEvent(EventPtr event)
{
	UInt16 formID;
	FormPtr form;
	Boolean handled = false;
	
	// Application event loop
	switch (event->eType)
	{
		case frmLoadEvent:
			// Handle form load events
			formID = event->data.frmLoad.formID;
			form = FrmInitForm(formID);
			FrmSetActiveForm(form);		
			switch (formID)
			{
				case frmTrip:
					// Set event handler for frmMain
					FrmSetEventHandler(form, (FormEventHandlerPtr) TripHandleEvent);
					handled = true;
					break;
				case frmNewTrip:
					// Set event handler for frmMain
					FrmSetEventHandler(form, (FormEventHandlerPtr) NewTripHandleEvent);
					handled = true;
					break;
				case frmMain:
					// Set event handler for frmMain
					FrmSetEventHandler(form, (FormEventHandlerPtr) MainFormHandleEvent);
					handled = true;
					break;
				case frmPrefs:
					// Set event handler for preferences form
					FrmSetEventHandler(form, (FormEventHandlerPtr) PreferencesHandleEvent);
					handled = true;
					break;
				case frmReplay:
					// Set event handler for replay form
					FrmSetEventHandler(form, (FormEventHandlerPtr) ReplayHandleEvent);
					handled = true;
					break;
				case frmAbout:
					// Set event handler for about form
					FrmSetEventHandler(form, (FormEventHandlerPtr) AboutHandleEvent);
					handled = true;
					break;
				default:
					break;
			}
			break;
		default:
			break;
	}

	return handled;
}


/***********************************************************************
 *
 * FUNCTION:	EventLoop
 *
 * DESCRIPTION:	The eventloop polls the event que for new events and
 *				dispatches them to the different event handlers.
 *
 * PARAMETERS:	 Nothing
 *					
 * RETURNED:	 Nothing
 *
 * REVISION HISTORY:
 *			Name		Date					Description
 *			----		----					-----------
 *
 ***********************************************************************/

static void EventLoop(void)
{
	Err error;
	EventType event;

	// Main event loop
	do {
		// Get next event
		EvtGetEvent(&event, 10);
		// PreprocessEvent is used to identify if a the hotsync button was pressed.
		// Handle event
		if (!SysHandleEvent(&event)) {
			if (!MenuHandleEvent(0, &event, &error)) {
				if (!ApplicationHandleEvent(&event))
					FrmDispatchEvent(&event);	        						
			}
		}
	}
	while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:	StopApplication
 *
 * DESCRIPTION:	this routine closes all open forms by calling
 *				FrmCloseAllForms()
 *
 * PARAMETERS:	 Nothing
 *					
 * RETURNED:	 Nothing
 *
 * REVISION HISTORY:
 *			Name		Date					Description
 *			----		----					-----------
 *
 ***********************************************************************/

static void StopApplication(void)
{
  //this should send a frmCloseEvent to all forms
	FrmCloseAllForms();
	// Release the font resource
	MemHandleUnlock(fontH);
	DmReleaseResource(fontH);
	SysSetAutoOffTime(autoOffTime);

  WinScreenMode(winScreenModeSetToDefaults,NULL,NULL,NULL,NULL);
  PabiloSavePrefs(&prefs);

	// Close the application's data file.
	if(PabiloDB) {
		DmCloseDatabase(PabiloDB);
		PabiloDB = NULL;
	}
	if(pabiloDBName) {
		MemPtrFree(pabiloDBName);
	}
}


/***********************************************************************
 *
 * FUNCTION:	PilotMain
 *
 * DESCRIPTION:	Main entrypoint for the application
 *
 * PARAMETERS:	cmd			Application Launchcode
 *				cmbBPB		Pointer to a structure that is 
 *							associated with the launch code. 		
 *				launchFlags	value providing extra information about 
 *							the launch
 *					
 * RETURNED:	Result of the launch
 *
 * REVISION HISTORY:
 *			Name		Date					Description
 *			----		----					-----------
 *
 ***********************************************************************/

UInt32 PilotMain(UInt16 cmd, void *cmdPBP, UInt16 launchFlags)
{
	Err error;
	
	switch (cmd) 
 	{
		case sysAppLaunchCmdNormalLaunch:
			// Application start code
			error = StartApplication();
			if (error)
				return error;
		
			// Maintain event loop
			EventLoop();

			// Stop application 
			StopApplication();		
			break;
		default:
			break;

	}

	return 0;
}

