/***************************************************************************
                          pabilotest  -  description
                             -------------------
    begin                : Tue May 21 2002
    copyright            : (C) 2002 by Michiel van Turnhout
    email                : info@qris.myweb.nl
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 * Test program for Pabilo palm bike log application                       *
 * It should be able to replace the Speedo AVR hardware. with 11.0592 MHz  *
 * The program receives a 'g' command from the palm and returns the				 *
 * 16 bit timercapture value and the 32bit wheelround value.							 *
 * in total 12bytes.                                                       *
 *                                                                         *
 * Took the example from the serial-programming-HowTo as a starting point  *
 *                                                                         *
 * ToDo:                                                                   *
 * create a relation between speed and timer1capture value.                *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>

#define BAUDRATE B115200
#define MODEMDEVICE "/dev/ttyS0"
#define _POSIX_SOURCE 1 /* POSIX compliant source */
#define FALSE 0
#define TRUE 1

volatile int running=FALSE;

struct termios oldtio,newtio;

int openSerial(char *device) {
  int fd;

  fd = open(device, O_RDWR | O_NOCTTY );
  if (fd <0) {perror(device); exit(-1); }

  tcgetattr(fd,&oldtio); /* save current port settings */

  bzero(&newtio, sizeof(newtio));
  newtio.c_cflag = BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;
  newtio.c_iflag = IGNPAR;
  newtio.c_oflag = 0;

  /* set input mode (non-canonical, no echo,...) */
  newtio.c_lflag = 0;

  newtio.c_cc[VTIME]    = 0;   /* inter-character timer unused */
  newtio.c_cc[VMIN]     = 1;   /* blocking read until 5 chars received */

  tcflush(fd, TCIFLUSH);
  tcsetattr(fd,TCSANOW,&newtio);

  return(fd);
}

void closeSerial(int fd) {
  tcsetattr(fd,TCSANOW,&oldtio);
}

/*
 * We should keep track of the speed and calculate the timercapture values
 * and the wheelrounds.
 */
main()
{
  int fd,c, res;
  struct termios oldtio,newtio;
  char buf[255];

  int speed = 1; // NOTE speed lower than 2 produce a 5char hex value and only the
                 // first 4 are interpreted evaluating to a higher speed!
  double timertic;
  double speedo_prescaler = 1024;
  double speedo_clock = 11059200;

  int whsz = 214;
	int wheelrounds = 0;
  int incval = 1;
	int speedo = 0;
	char spd[] = "0000";
	char whr[] = "00000000";

  //calculate the tic update rate for timer1 in the AVR
  timertic = (double) speedo_prescaler / speedo_clock;

  fd = openSerial(MODEMDEVICE);

  while (TRUE) {       /* loop for input */
		wheelrounds+=incval;
    if(speed++ > 70) speed=1;
    incval=speed/4;
    //calculate from speed to timer 1 capture value
    //var1 = (double) speed * 27.7778;
    speedo = ((double) whsz / ((double) speed * 27.7778)) / timertic;

    res = read(fd,buf,1);   /* returns after 1 char have been input */
    buf[res]=0;               /* so we can printf... */
    printf(":%s:%d\n", buf, res);
		if (buf[0]=='g') {
			sprintf(spd,"%04x",speedo);
			sprintf(whr,"%08x",wheelrounds);
			sprintf(buf,"%s%s",spd,whr);
			write(fd,buf,12);
      printf("speed %d capture %04x\n",speed,speedo);
		}else
      if (buf[0]=='r') {
        speed = 1 ;
        wheelrounds = 0;
      }
	}
  closeSerial(fd);
 }
